from .task import (
    SemanticTriplet,
    TextRetrieval,
    TextMatching,
    TextClassification,
)

__all__ = [
    "SemanticTriplet",
    "TextRetrieval",
    "TextMatching",
    "TextClassification",
]
